#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>

//Räsitabeli ämber
typedef struct bucket 
{
	float key;
	struct bucket *next;
} bucket;

//Räsitabel
typedef struct 
{
	int k; 
	int n; 
	bucket **buckets;	 
} hash_table;

unsigned long hash(void *data, int length);
void hash_table_init(hash_table *table, int k);
void bucket_init(bucket **b, float key);
void hash_table_insert(hash_table *table, float key);	 
int hash_table_contains(hash_table *table, float key);	//Küsi, kas väärtus on olemas?
void hash_table_delete(hash_table *table);

int main()
{
	hash_table table;
	hash_table_init(&table, 1000);
	hash_table_insert(&table, 1.0);
	hash_table_insert(&table, 2.0);
	hash_table_insert(&table, 3.0);
	hash_table_insert(&table, 4.0);
	hash_table_insert(&table, 5.0);
	printf("is it exist 1.0: %d\n", hash_table_contains(&table, 1.0));
	printf("  is it exist 1.2: %d\n", hash_table_contains(&table, 1.2));
	printf(" is it exist 3.0: %d\n", hash_table_contains(&table, 3.0));
	return 0;
}

unsigned long hash(void *data, int length)
{
    unsigned long hash = 5381;
    int i;
    for (i = 0; i < length; i++)
    {
		hash = ((hash << 5) + hash) + ((char *)data)[i]; /* hash * 33 + c */	
	}
    return hash;
}

void hash_table_init(hash_table *table, int k)
{
	table->buckets = malloc(sizeof(*table->buckets) * k);
	table->k = k;
	table->n = 0;
}

void bucket_init(bucket **b, float key)
{
	*b = malloc(sizeof(bucket));
	(*b)->key = key;
	(*b)->next = NULL;
}

void hash_table_insert(hash_table *table, float key)
{
	unsigned long k = hash(&key, sizeof(key)) % table->k;
	if (table->buckets[k] == NULL)
	{
		bucket_init(table->buckets + k, key);
		return;
	}
	bucket *b = table->buckets[k];
	while (1)
	{
		if (b->key == key)
		{
			return;
		}
		if (b->next == NULL)
		{
			bucket_init(&b->next, key);
			return;
		}
		b = b->next;
	}
}

int hash_table_contains(hash_table *table, float key)
{
	unsigned long k = hash(&key, sizeof(key)) % table->k;
	if (table->buckets[k] == NULL)
	{
		return 0;
	}
	bucket *b = table->buckets[k];
	while (1)
	{
		if (b->key == key)
		{
			return 1;
		}
		if (b->next == NULL)
		{
			return 0;
		}
		b = b->next;
	}
}

void hash_table_delete(hash_table *table)
{
	int i;
	bucket *b;
	bucket *b2;
	for (i = 0; i < table->k; i++)
	{
		b = table->buckets[i];
		while (1)
		{
			b2 = b;
			b = b->next;
			free(b2);
		}
	}
	free(table->buckets);
}
